﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Moq;
using System;

namespace Property.Test
{
    public abstract class CallbackReloaderDecoratorPrototypeTestInfrastructure<T> : ReloaderDecoratorPrototypeTestInfrastructure<T>
    {
        private Mock<ITimerPrototype> TimerPrototypeMockState;
        protected Mock<ITimerPrototype> TimerPrototypeMock
        {
            get
            {
                if (TimerPrototypeMockState == null)
                {
                    TimerPrototypeMockState = new Mock<ITimerPrototype>(MockBehavior.Strict);
                    TimerPrototypeMockState.Setup(o => o.Create()).Returns(TimerMock.Object);
                }
                return TimerPrototypeMockState;
            }
        }

        private Mock<ITimer> TimerMockState;
        protected Mock<ITimer> TimerMock
        {
            get
            {
                if (TimerMockState == null)
                {
                    TimerMockState = new Mock<ITimer>(MockBehavior.Strict);
                    TimerMockState.Setup(o => o.Start()).Raises(o => o.Tick += null, new EventArgs());
                    TimerMockState.Setup(o => o.Stop());
                    TimerMockState.Setup(o => o.Dispose());
                }
                return TimerMockState;
            }
        }

        private Mock<ITimerPrototype> NullTimerPrototypeMockState;
        protected Mock<ITimerPrototype> NullTimerPrototypeMock
        {
            get
            {
                if (NullTimerPrototypeMockState == null)
                {
                    NullTimerPrototypeMockState = new Mock<ITimerPrototype>(MockBehavior.Strict);
                    NullTimerPrototypeMockState.Setup(o => o.Create()).Returns<ITimer>(null);
                }
                return NullTimerPrototypeMockState;
            }
        }

        //=//

        protected virtual void VerifyTimerDispose()
        {
            TimerMock.Verify(o => o.Dispose(), Times.Once());
        }

        protected virtual void VerifyTimerStart(int times)
        {
            TimerMock.Verify(o => o.Start(), Times.Exactly(times));
        }

        protected virtual void VerifyTimerStop(int times)
        {
            TimerMock.Verify(o => o.Stop(), Times.Exactly(times));
        }

        protected override void VerifyDispose()
        {
            VerifyTimerDispose();
            base.VerifyDispose();
        }
    }
}
