﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Moq;
using System;
using System.ComponentModel;

namespace Property.Test
{
    public abstract class CallbackReloaderPrototypeTestInfrastructure<T> : ReloaderPrototypeTestInfrastructure<T>
    {
        private Mock<IFunctionWorkerPrototype<T>> FunctionWorkerPrototypeMockState;
        protected Mock<IFunctionWorkerPrototype<T>> FunctionWorkerPrototypeMock
        {
            get
            {
                if (FunctionWorkerPrototypeMockState == null)
                {
                    FunctionWorkerPrototypeMockState = new Mock<IFunctionWorkerPrototype<T>>(MockBehavior.Strict);
                    FunctionWorkerPrototypeMockState.Setup(o => o.Create(It.Is<Func<T>>(f => f == GetValue)))
                        .Returns(WorkerMock.Object);
                }
                return FunctionWorkerPrototypeMockState;
            }
        }

        private Mock<IWorker> WorkerMockState;
        protected Mock<IWorker> WorkerMock
        {
            get
            {
                if (WorkerMockState == null)
                {
                    WorkerMockState = new Mock<IWorker>(MockBehavior.Strict);
                    WorkerMockState.Setup(o => o.Dispose());
                    WorkerMockState.Setup(o => o.IsBusy).Returns(() => TestIsBusy);
                }
                return WorkerMockState;
            }
        }

        protected virtual bool TestIsBusy { get { return true; } }

        private Mock<IFunctionWorkerPrototype<T>> NullFunctionWorkerPrototypeMockState;
        protected Mock<IFunctionWorkerPrototype<T>> NullFunctionWorkerPrototypeMock
        {
            get
            {
                if (NullFunctionWorkerPrototypeMockState == null)
                {
                    NullFunctionWorkerPrototypeMockState = new Mock<IFunctionWorkerPrototype<T>>(MockBehavior.Strict);
                    NullFunctionWorkerPrototypeMockState.Setup(o => o.Create(It.IsAny<Func<T>>())).Returns<IWorker>(null);
                }
                return NullFunctionWorkerPrototypeMockState;
            }
        }

        protected virtual void SetupReload()
        {
            WorkerMockState.Setup(o => o.RunWorkerAsync()).Raises(o => o.RunWorkerCompleted += null, () => new RunWorkerCompletedEventArgs(GetValue(), null, false));
        }

        protected virtual void SetupReloadException()
        {
            WorkerMockState.Setup(o => o.RunWorkerAsync()).Raises(o => o.RunWorkerCompleted += null, () => new RunWorkerCompletedEventArgs(null, new ApplicationException(), false));
        }

        //=//

        protected virtual void VerifyDispose()
        {
            WorkerMock.Verify(o => o.Dispose(), Times.AtLeastOnce());
        }
    }
}
