﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Moq;

namespace Property.Test
{
    public abstract class FunctionWorkerDecoratorPrototypeTestInfrastructure<T> : FunctionWorkerPrototypeTestInfrastructure<T>
    {
        private Mock<IFunctionWorkerPrototype<T>> FunctionWorkerPrototypeMockState;
        protected Mock<IFunctionWorkerPrototype<T>> FunctionWorkerPrototypeMock
        {
            get
            {
                if (FunctionWorkerPrototypeMockState == null)
                {
                    FunctionWorkerPrototypeMockState = new Mock<IFunctionWorkerPrototype<T>>(MockBehavior.Strict);
                    FunctionWorkerPrototypeMockState.Setup(o => o.Create(It.IsAny<Func<T>>())).Returns(WorkerMock.Object);
                }
                return FunctionWorkerPrototypeMockState;
            }
        }

        private Mock<IFunctionWorkerPrototype<T>> NullFunctionWorkerPrototypeMockState;
        protected Mock<IFunctionWorkerPrototype<T>> NullFunctionWorkerPrototypeMock
        {
            get
            {
                if (NullFunctionWorkerPrototypeMockState == null)
                {
                    NullFunctionWorkerPrototypeMockState = new Mock<IFunctionWorkerPrototype<T>>(MockBehavior.Strict);
                    NullFunctionWorkerPrototypeMockState.Setup(o => o.Create(It.IsAny<Func<T>>())).Returns<IWorker>(null);
                }
                return NullFunctionWorkerPrototypeMockState;
            }
        }

        private Mock<ITimerPrototype> TimerPrototypeMockState;
        protected Mock<ITimerPrototype> TimerPrototypeMock
        {
            get
            {
                if (TimerPrototypeMockState == null)
                {
                    TimerPrototypeMockState = new Mock<ITimerPrototype>(MockBehavior.Strict);
                    TimerPrototypeMockState.Setup(o => o.Create()).Returns(TimerMock.Object);
                }
                return TimerPrototypeMockState;
            }
        }

        private Mock<ITimer> TimerMockState;
        protected Mock<ITimer> TimerMock
        {
            get
            {
                if (TimerMockState == null)
                {
                    TimerMockState = new Mock<ITimer>(MockBehavior.Strict);
                    TimerMockState.Setup(o => o.Start());
                    TimerMockState.Setup(o => o.Stop());
                    TimerMockState.Setup(o => o.Dispose());
                }
                return TimerMockState;
            }
        }

        private Mock<ITimerPrototype> NullTimerPrototypeMockState;
        protected Mock<ITimerPrototype> NullTimerPrototypeMock
        {
            get
            {
                if (NullTimerPrototypeMockState == null)
                {
                    NullTimerPrototypeMockState = new Mock<ITimerPrototype>(MockBehavior.Strict);
                    NullTimerPrototypeMockState.Setup(o => o.Create()).Returns<ITimer>(null);
                }
                return NullTimerPrototypeMockState;
            }
        }

        //=//

        protected override void VerifyWorkerCreate()
        {
            FunctionWorkerPrototypeMockState.Verify(o => o.Create(It.IsAny<Func<T>>()), Times.Once());
        }

        protected virtual void VerifyWorkerCall(int times)
        {
            WorkerMock.Verify(o => o.RunWorkerAsync(), Times.Exactly(times));
        }

        protected virtual void VerifyTimerStart(int times)
        {
            TimerMock.Verify(o => o.Start(), Times.Exactly(times));
        }

        protected virtual void VerifyTimerStop(int times)
        {
            TimerMock.Verify(o => o.Stop(), Times.Exactly(times));
        }

        protected virtual void VerifyTimerDispose()
        {
            TimerMock.Verify(o => o.Dispose(), Times.Once()); 
        }
    }
}
