﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Property.Workers;
using System.Collections.Generic;
using System.ComponentModel;

namespace Property.Test.Workers
{
    [TestClass]
    public class TimeoutFunctionWorkerDecoratorPrototypeTest : FunctionWorkerDecoratorPrototypeTestInfrastructure<object>
    {
        protected override IEnumerable<string> TestEventHandlerCallStrings
        { get { return new[] { "RunWorkerCompleted" }; } }

        private readonly object _TestValue = new object();
        protected override object TestValue { get { return _TestValue; } }

        private bool TestIsBusyState = false;
        protected override bool TestIsBusy { get { return TestIsBusyState; } }

        protected override Type TestException { get { return typeof(TimeoutException); } }

        [TestMethod]
        public void TimeoutFunctionWorkerDecoratorNoTimeoutTest()
        {
            var sutDecorator = new TimeoutFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, TimerPrototypeMock.Object).Create(() => null);

            sutDecorator.RunWorkerCompleted += RunWorkerCompleted;

            sutDecorator.RunWorkerAsync();
            VerifyTimerStart(1);
            VerifyTimerStop(1);

            WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(TestValue, null, false));

            VerifyTimerStart(1);
            VerifyTimerStop(2);

            VerifyCompleted();
            VerifyRunWorkerAsync();
        }

        [TestMethod]
        public void TimeoutFunctionWorkerDecoratorTimeoutTest()
        {
            var sutDecorator = new TimeoutFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, TimerPrototypeMock.Object).Create(() => null);

            sutDecorator.RunWorkerCompleted += RunWorkerCompleted;

            sutDecorator.RunWorkerAsync();
            VerifyTimerStart(1);
            VerifyTimerStop(1);

            TimerMock.Raise(o => o.Tick += null, new EventArgs());
            WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(TestValue, null, false));

            VerifyTimerStart(1);
            VerifyTimerStop(3);

            VerifyCompletedException();
            VerifyRunWorkerAsync();
        }

        [TestMethod]
        public void TimeoutFunctionWorkerDecoratorDisposeTest()
        {
            var sutDecorator = new TimeoutFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, TimerPrototypeMock.Object).Create(() => null);
            var reference = new WeakReference(sutDecorator, true);

            sutDecorator.RunWorkerCompleted += RunWorkerCompleted;
            sutDecorator.RunWorkerAsync();
            sutDecorator.Dispose();
            sutDecorator = null;
            GC.Collect();

            Assert.IsNull(reference.Target);
            VerifyDispose();
            VerifyTimerDispose();
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TimeoutFunctionWorkerDecoratorNullTimerPrototypeTest()
        {
            new TimeoutFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, null).Create(() => null);
        }

        [TestMethod]
        [ExpectedException(typeof(NotSupportedException))]
        public void TimeoutFunctionWorkerDecoratorNullTimerTest()
        {
            new TimeoutFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, NullTimerPrototypeMock.Object).Create(() => null);
        }
    }
}
