﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace Property.Test
{
    public abstract class WorkerTestInfrastructure<T>
    {
        protected virtual T TestValue { get { return default(T); } }

        protected virtual Type TestException { get { return typeof(ApplicationException); } }

        protected virtual IEnumerable<string> TestEventHandlerCallStrings
        { get { return new[] { "DoWork", "RunWorkerCompleted"}; } }
        private IEnumerable<string> ResultEventHandlerCallStrings = Enumerable.Empty<string>();

        protected virtual IEnumerable<Type> TestExceptions
        { get { return new[] { TestException }; } }
        private IEnumerable<Type> ResultExceptions = Enumerable.Empty<Type>();

        protected virtual IEnumerable<T> TestValues
        { get { return new[] { TestValue }; } }
        private IEnumerable<T> ResultValues = Enumerable.Empty<T>();

        protected virtual void DoWork(object sender, DoWorkEventArgs e)
        {
            ResultEventHandlerCallStrings = ResultEventHandlerCallStrings.Concat(new[] { "DoWork" });
            e.Result = TestValue;
        }

        protected virtual void DoWorkException(object sender, DoWorkEventArgs e)
        {
            DoWork(sender, e);
            throw new ApplicationException();
        }

        protected virtual void RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ResultEventHandlerCallStrings = ResultEventHandlerCallStrings.Concat(new[] { "RunWorkerCompleted" });

            if (e.Error != null)
                ResultExceptions = ResultExceptions.Concat(new[] { e.Error.GetType() });
            else
                ResultValues = ResultValues.Concat(new[] { (T)e.Result });
        }

        protected virtual void TestHandler(Exception e)
        {
            ResultEventHandlerCallStrings = ResultEventHandlerCallStrings.Concat(new[] { "TestHandler" });
            ResultExceptions = ResultExceptions.Concat(new[] { e.GetType() });
        }

        //=//

        protected virtual void VerifyCompleted()
        {
            new[] { ResultEventHandlerCallStrings.SequenceEqual(TestEventHandlerCallStrings), 
                    ResultValues.SequenceEqual(TestValues) }
                    .Where(o => !o)
                    .ToList()
                    .ForEach(o => { throw new Exception(); });
        }

        protected virtual void VerifyCompletedException()
        {
            new[] { ResultExceptions.SequenceEqual(TestExceptions), 
                    ResultEventHandlerCallStrings.SequenceEqual(TestEventHandlerCallStrings)}
                    .Where(o => !o)
                    .ToList()
                    .ForEach(o => { throw new Exception(); });
        }
    }
}
