﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Property.Reloaders;

namespace Property.Test.Reloaders
{
    [TestClass]
    public class SyncReloaderPrototypeTest : ReloaderPrototypeTestInfrastructure<object>
    {
        private readonly object _TestValue = new object();
        protected override object TestValue
        {
            get { return _TestValue; }
        }

        [TestMethod]
        public void SyncReloaderReloadTest()
        {
            var sutReloader = new SyncReloaderPrototype<object>().Create(GetValue, ApplyValue, ApplyStatus);

            sutReloader.Go();

            VerifyReload();
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SyncReloaderNullGetValueTest()
        {
            new SyncReloaderPrototype<object>().Create(null, ApplyValue, ApplyStatus);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SyncReloaderNullApplyValueTest()
        {
            new SyncReloaderPrototype<object>().Create(GetValue, null, ApplyStatus);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void SyncReloaderNullApplyStatusTest()
        {
            new SyncReloaderPrototype<object>().Create(GetValue, ApplyValue, null);
        }

        [TestMethod]
        public void SyncReloaderDisposeTest()
        {
            var sutReloader = new SyncReloaderPrototype<object>().Create(GetValue, ApplyValue, ApplyStatus);
            var reference = new WeakReference(sutReloader, true);
            sutReloader.Go();
            sutReloader.Dispose();
            sutReloader = null;
            GC.Collect();
            Assert.IsNull(reference.Target);
        }
    }
}
