﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Property.Objects;
using System;
using System.Collections.Generic;

namespace Property.Test.Objects
{
    [TestClass]
    public class CheckInputPrototypePropertyTest : CheckPropertyPrototypeTestInfrastructure<object>
    {
        protected override IEnumerable<string> TestEventStrings
        { get { return new[] { 
            "Changing'Input'", 
            "Changed'Input'", 
            "Changing'IsLoading'", 
            "Changed'IsLoading'", 
            "Changing'Value'", 
            "Changed'Value'",
            "Changed'Error'"}; } }

        [TestMethod]
        public void CheckInputPropertyReloadTest()
        {
            var testValue = new object();

            var sutProperty = new CheckInputPropertyPrototype<object>(
                        ReloaderPrototypeMock.Object, o => TestErrorString).Create(null);

            SetupCheck(sutProperty, "Input");
            SetupEvents(sutProperty);

            Assert.AreEqual(default(object), sutProperty.Input);
            Assert.AreEqual(default(object), sutProperty.Value);
            Assert.AreEqual(default(object), sutProperty.DebugValue);
            Assert.AreEqual(default(bool), sutProperty.IsLoading);

            sutProperty.Input = testValue;

            Assert.AreEqual(testValue, sutProperty.Value);
            Assert.AreEqual(testValue, sutProperty.DebugValue);
            Assert.AreEqual(TestStatus, sutProperty.IsLoading);

            VerifyCheck();
            VerifyEvents();
            VerifyReload();
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void CheckInputPropertyNullValidationTest()
        {
            new CheckInputPropertyPrototype<object>(
                        ReloaderPrototypeMock.Object, null).Create(null);
        }
    }
}
