﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading;

namespace Showcase.LoadAsYouScroll
{
    // Contains methods that can be used to unit-test ScrollList.
    // In real life: move to a separate unit-test project.
    public static class ScrollListCheck
    {
        private const int ValuesOnPage = 2;

        public static CheckResult CreateRunDisposeCheck<T>(int load)
        {
            ScrollList<T>[] lists = new ScrollList<T>[load];
            WeakReference[] references = new WeakReference[load];
            var testRange = Enumerable.Range(0, load).ToList();
            T tval;

            var stopwatch = new Stopwatch();
            stopwatch.Start();

            testRange.ForEach(o =>
            {
                lists[o] = new ScrollList<T>(GetIt<T>, true);
                references[o] = new WeakReference(lists[o], true);
                tval = lists[o].List.Value.FirstOrDefault();
                lists[o].Command.Execute(null);
                lists[o].Dispose();
            });

            stopwatch.Stop();
            var performanceResult = stopwatch.Elapsed;

            testRange.ForEach(o => lists[o] = null);
            GC.Collect();

            var memoryResult = !testRange.Any(o => references[o].Target != null);

            return new CheckResult(performanceResult, memoryResult);
        }

        public static bool OperationCheck<T>()
        {
            var result = Enumerable.Empty<T>();
            var list = new ScrollList<T>(GetIt<T>, true);
            list.List.PropertyChanged += (o, e) => result = e.PropertyName == "Value" ? list.List.Value : result;

            var test = list.List.Value;
            list.Command.Execute(null);
            return result.Count() == ValuesOnPage * 2;
        }

        private static IEnumerable<T> GetIt<T>(int page)
        {
            return Enumerable.Repeat(default(T), ValuesOnPage);
        }
    }
}
