﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Property;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Showcase.ProgressCounter
{
    // Represents a counter with reset.
    // The first Go call after a Reset command 
    // will set the counter to minimum. 
    public class Counter : ICallDriven, IDisposable
    {
        /// <param name="minimum">Tell me how to get the progress minimum.</param>
        /// <param name="maximum">Tell me how to get the progress maximum.</param>
        /// <param name="step">Tell me how to get the progress step.</param>
        public Counter(Func<int> minimum, Func<int> maximum, Func<int> step)
        {
            var reset = new Actuator();

            Func<int?, int?> count = o =>
            {
                if (!o.HasValue || reset.Power.Value) return minimum();

                var res = o + step();
                return res < minimum() ? minimum() : 
                    res > maximum() ? maximum() : 
                    res;
            };

            var counter = Reloadable<int?>.On().Each().Call(count).Create(null);
            var output = Reloadable<int>.On().Each().Call(o => counter.Value.Value).Create(minimum());
            var isReset = Reloadable<bool>.On().Each().Get(_ => reset.Power.Value).Create();

            var counterToOutput = counter.OnChanged((Action)output.Go);
            var counterToReset = counter.OnChanged((Action)reset.Stop);

            _IsReset = isReset;
            _Output = output;
            _Reset = reset.Start;
            _Go = counter.Go;

            // Disposables.
            _ToDispose = new[] { counterToOutput, counterToReset };
        }

        private readonly IProperty<int> _Output;
        public IProperty<int> Output { get { return _Output; } }

        private readonly Action _Reset;
        public void Reset() { _Reset(); }

        private readonly Action _Go;
        public void Go() { _Go(); }

        private readonly IProperty<bool> _IsReset;
        public IProperty<bool> IsReset { get { return _IsReset; } }

        //=//

        private readonly IList<IDisposable> _ToDispose;

        public void Dispose()
        {
            _ToDispose.ToList().ForEach(o => o.Dispose()); ;
        }
    }
}
