﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Property.Sequences;
using System.Reactive.Linq;
using System.Reactive.Concurrency;
using System;

namespace Property.Windows
{
    /// <summary>
    /// Contains words that extend the description 
    /// of the sequence-based property value loading process.
    /// </summary>
    public static class SequenceConveyorExtensionsReactiveWindows
    {
        /// <summary>
        /// <para>... asynchronously ...</para>
        /// <para>New values will be loaded on a new thread and applied on Dispatcher thread.</para>
        /// </summary>
        /// <param name="testMode">Tell me if I have to ignore this word.</param>
        public static ISequenceConveyor<T> Async<T>(this ISequenceConveyor<T> source, bool testMode = false)
        {
            return testMode ? source : source.Modify(o => o
                .SubscribeOn(NewThreadScheduler.Default)
                .ObserveOn(DispatcherScheduler.Current));
        }

        /// <summary>
        /// <para>... if the call passes through the throttle (when) ...</para>
        /// <para>( ! ) Don't forget to dispose of what you create with this word.</para>
        /// </summary>
        /// <param name="dueTime">Tell me how long I should
        /// wait for another call before letting this one pass.</param>
        /// <param name="testMode">Tell me if I have to ignore this word.</param>
        public static IReloaderPrototype<T> Throttle<T>(this ISequenceConveyor<T> source, TimeSpan dueTime, bool testMode = false)
        {
            return source.Each().Throttle(dueTime, testMode);
        }
    }
}
