﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Moq;

namespace Property.Test
{
    public abstract class FunctionWorkerPrototypeTestInfrastructure<T> : WorkerTestInfrastructure<T>
    {
        private Mock<IWorker> WorkerMockState;
        protected Mock<IWorker> WorkerMock
        {
            get
            {
                if (WorkerMockState == null)
                {
                    WorkerMockState = new Mock<IWorker>(MockBehavior.Strict);
                    WorkerMockState.Setup(o => o.RunWorkerAsync());
                    WorkerMockState.Setup(o => o.Dispose());
                    WorkerMockState.Setup(o => o.IsBusy).Returns(() => TestIsBusy);
                }
                return WorkerMockState;
            }
        }

        protected virtual bool TestIsBusy { get { return true; } }

        private Mock<IWorkerPrototype> WorkerPrototypeMockState;
        protected Mock<IWorkerPrototype> WorkerPrototypeMock
        {
            get
            {
                if (WorkerPrototypeMockState == null)
                {
                    WorkerPrototypeMockState = new Mock<IWorkerPrototype>(MockBehavior.Strict);
                    WorkerPrototypeMockState.Setup(o => o.Create()).Returns(WorkerMock.Object);

                }
                return WorkerPrototypeMockState;
            }
        }

        private Mock<IWorkerPrototype> NullWorkerPrototypeMockState;
        protected Mock<IWorkerPrototype> NullWorkerPrototypeMock
        {
            get
            {
                if (NullWorkerPrototypeMockState == null)
                {
                    NullWorkerPrototypeMockState = new Mock<IWorkerPrototype>(MockBehavior.Strict);
                    NullWorkerPrototypeMockState.Setup(o => o.Create()).Returns<IWorker>(null);
                }
                return NullWorkerPrototypeMockState;
            }
        }

        //=//

        protected virtual void VerifyWorkerCreate()
        {
            WorkerPrototypeMockState.Verify(o => o.Create(), Times.Once());
        }

        protected virtual void VerifyRunWorkerAsync(int times)
        {
            VerifyWorkerCreate();
            WorkerMockState.Verify(o => o.RunWorkerAsync(), Times.Exactly(times));
        }

        protected virtual void VerifyRunWorkerAsync()
        {
            VerifyRunWorkerAsync(1);
        }

        protected virtual void VerifyIsBusy()
        {
            WorkerMockState.Verify(o => o.IsBusy, Times.Once());
        }

        protected virtual void VerifyDispose()
        {
            WorkerMockState.Verify(o => o.Dispose(), Times.Once());
        }

        protected virtual void VerifyDisposeCreate(int times)
        {
            WorkerPrototypeMock.Verify(o => o.Create(), Times.Exactly(times));
            WorkerMockState.Verify(o => o.Dispose(), Times.Exactly(times));
        }
    }
}
