﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using System.Collections.Generic;
using System.Linq;

namespace Property.Test
{
    public abstract class ReloaderPrototypeTestInfrastructure<T>
    {
        private IEnumerable<string> ResultMethodCallStrings = Enumerable.Empty<string>();

        protected virtual IEnumerable<string> TestMethodCallStrings
        { get { return new[] { "ApplyStatus", "GetValue", "ApplyStatus", "ApplyValue" }; } }

        protected virtual T TestValue { get { return default(T); } }

        protected virtual T GetValue()
        {
            ResultMethodCallStrings = ResultMethodCallStrings.Concat(new[] { "GetValue" });
            return TestValue;
        }

        private IEnumerable<T> ResultValues = Enumerable.Empty<T>();

        protected virtual IEnumerable<T> TestValues
        { get { return new[] { TestValue }; } }

        protected virtual void ApplyValue(T value)
        {
            ResultMethodCallStrings = ResultMethodCallStrings.Concat(new[] { "ApplyValue" });
            ResultValues = ResultValues.Concat(new[] { value });
        }

        private IEnumerable<bool> ResultStatuses = Enumerable.Empty<bool>();

        protected virtual IEnumerable<bool> TestStatuses
        { get { return new[] { true, false }; } }

        protected virtual void ApplyStatus(bool status)
        {
            ResultMethodCallStrings = ResultMethodCallStrings.Concat(new[] { "ApplyStatus" });
            ResultStatuses = ResultStatuses.Concat(new[] { status });
        }

        //=//

        protected void VerifyReload()
        {
            new[] { ResultStatuses.SequenceEqual(TestStatuses), 
                    ResultValues.SequenceEqual(TestValues),
                    ResultMethodCallStrings.SequenceEqual(TestMethodCallStrings) }
                .Where(o => !o)
                .ToList()
                .ForEach(o => { throw new Exception(); });
        }
    }
}
