﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using System.Reactive.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Property.Reloaders;

namespace Property.Test.Reloaders
{
    [TestClass]
    public class StateFilterReloaderDecoratorPrototypeTest : ReloaderDecoratorPrototypeTestInfrastructure<object>
    {
        [TestMethod]
        public void StateFilterReloaderDecoratorReloadTest()
        {
            var sutDecorator = new StateFilterReloaderDecoratorPrototype<object, int>(ReloaderPrototypeMock.Object, o => o + 1, o => o.Where(i => i % 2 == 0))
                .Create(() => null, _ => { }, __ => { });
            sutDecorator.Go();
            VerifyReload(1);
            sutDecorator.Go();
            VerifyReload(1);
            sutDecorator.Go();
            VerifyReload(2);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void StateFilterReloaderDecoratorNullGetFilterStateTest()
        {
            new StateFilterReloaderDecoratorPrototype<object, int>(ReloaderPrototypeMock.Object, null, o => o)
                .Create(() => null, _ => { }, __ => { });
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void StateFilterReloaderDecoratorNullFilterTest()
        {
            new StateFilterReloaderDecoratorPrototype<object, int>(ReloaderPrototypeMock.Object, o => o, null)
                .Create(() => null, _ => { }, __ => { });
        }

        [TestMethod]
        public void StateFilterReloaderDecoratorDisposeTest()
        {
            var sutDecorator = new StateFilterReloaderDecoratorPrototype<object, int>(ReloaderPrototypeMock.Object, o => o + 1, o => o.Where(i => i % 2 == 1))
                .Create(() => null, _ => { }, __ => { });
            var reference = new WeakReference(sutDecorator, true);
            sutDecorator.Go();
            sutDecorator.Go();
            sutDecorator.Dispose();
            sutDecorator = null;
            GC.Collect();
            Assert.IsNull(reference.Target);
        }
    }
}
