﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;

namespace Showcase.LoadAsYouScroll
{
    public class FireWhenScrolledBehaviour : Behavior<ItemsControl>
    {
        #region Command

        public static readonly DependencyProperty CommandProperty =
             DependencyProperty.Register("Command", typeof(ICommand),
             typeof(FireWhenScrolledBehaviour));

        public ICommand Command
        {
            get { return (ICommand)GetValue(CommandProperty); }
            set { SetValue(CommandProperty, value); }
        }

        #endregion Command

        private ScrollViewer ScrollViewerState;

        protected override void OnAttached()
        {
            AssociatedObject.Loaded += OnLoaded;
            AssociatedObject.Unloaded += OnUnLoaded;
        }

        protected override void OnDetaching()
        {
            AssociatedObject.Loaded -= OnLoaded;
            AssociatedObject.Unloaded -= OnUnLoaded;
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Decorator border = VisualTreeHelper.GetChild(AssociatedObject, 0) as Decorator;
            ScrollViewerState = border.Child as ScrollViewer;

            ScrollViewerState.ScrollChanged += ScrollViewerState_ScrollChanged;
            ScrollViewerState_ScrollChanged(this, null);
        }

        private void OnUnLoaded(object sender, RoutedEventArgs e)
        {
            ScrollViewerState.ScrollChanged -= ScrollViewerState_ScrollChanged;
        }

        void ScrollViewerState_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            if (ScrollViewerState.ContentVerticalOffset > ScrollViewerState.ExtentHeight - ScrollViewerState.ViewportHeight-1)
            {
                new[] { Command }
                    .OfType<ICommand>()
                    .Where(o => o.CanExecute(null))
                    .ToList().ForEach(o => o.Execute(null));
            }
        }   
    }
}
