﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace Showcase.LoadByWire
{
    // You'll need something like this for validation.
    // The question is where you want to place it... Oh, and how you'll implement it...
    public static class ValidationService
    {
        public static string GetPassportNrError(string passportNr)
        {
            if (string.IsNullOrWhiteSpace(passportNr)) return "Empty";
            if (passportNr.Length > 30) return "TooLong";

            return null;
        }

        public static string GetFirstNameError(string firstName)
        {
            if (string.IsNullOrWhiteSpace(firstName)) return "Empty";
            if (firstName.Length > 30) return "TooLong";
            if (!Regex.IsMatch(firstName, @"^([ \u00c0-\u01ffa-zA-Z'\-])+$")) return "IllegalCharacters";

            return null;
        }

        public static string GetLastNameError(string lastName)
        {
            if (string.IsNullOrWhiteSpace(lastName)) return "Empty";
            if (lastName.Length > 30) return "TooLong";
            if (!Regex.IsMatch(lastName, @"^([ \u00c0-\u01ffa-zA-Z'\-])+$")) return "IllegalCharacters";

            return null;
        }

        public static string GetCompanyError(string company)
        {
            if (string.IsNullOrWhiteSpace(company)) return "Empty";
            if (company.Length > 30) return "TooLong";

            return null;
        }

        public static string GetEmailError(string email)
        {
            if (string.IsNullOrWhiteSpace(email)) return "Empty";

            // Copied from Phil Haack's site
            // http://haacked.com/archive/2007/08/21/i-knew-how-to-validate-an-email-address-until-i.aspx
            // Is used here for demonstration purposes only

            string pattern = @"^(?!\.)(""([^""\r\\]|\\[""\r\\])*""|"
                + @"([-a-z0-9!#$%&'*+/=?^_`{|}~]|(?<!\.)\.)*)(?<!\.)"
                + @"@[a-z0-9][\w\.-]*[a-z0-9]\.[a-z][a-z\.]*[a-z]$";

            if (!Regex.IsMatch(email, pattern)) return "IllegalFormat";

            return null;
        }
    }
}
