﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Property;
using Property.Windows.Forms;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Threading;
using System.Windows.Forms;

namespace Showcase.ProgressCounter
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            Func<int> minimum = () => MainProgressBar.Minimum;
            Func<int> maximum = () => MainProgressBar.Maximum;
            Func<int> step = () => MainProgressBar.Step;
            
            var counter = new Counter(minimum, maximum, step);
            var allowWork = new Actuator();

            // Declare shortcuts.
            Func<bool> workAllowed = () => allowWork.Power.Value;
            Func<bool> workNotDone = () => counter.Output.Value < maximum();
            Func<bool> needReset = () => counter.IsReset.Value;
            Func<bool> needWork = () => needReset() || (workNotDone() && workAllowed());

            Func<int, int> work = o => 
            {
                if (needReset()) return 0;
                Thread.Sleep(1000);
                return o + 1;
            };

            var doWork = Reloadable<int>.On().Worker().Conditional(needWork).Call(work).Create();
            var onDoWork = doWork.OnChanged(() => counter, () => doWork);

            // Data binding Windows Forms styled.
            MainProgressBar.DataBindings.Add("Value", counter.Output, "Value");
            WorkProgressLabel.DataBindings.Add("Text", doWork, "Value");
            ProgressCountLabel.DataBindings.Add("Text", counter.Output, "Value");

            // Assign fields.
            _AllowWork = allowWork;
            _Counter = counter;
            _DoWork = doWork.Go;

            // Add disposables to the cleanup list.
            _ToDispose = new[] { doWork, onDoWork };
        }

        //=//

        private readonly Action _DoWork;
        private readonly Actuator _AllowWork;
        private readonly Counter _Counter;

        private void StartButton_Click(object sender, EventArgs e)
        {
            _AllowWork.Start();
            _Counter.Reset();
            _DoWork();
        }

        private void AbortButton_Click(object sender, EventArgs e)
        {
            _AllowWork.Stop();
            _Counter.Reset();
            _DoWork();
        }

        private void ResumeButton_Click(object sender, EventArgs e)
        {
            _AllowWork.Start();
            _DoWork();
        }

        private void PauseButton_Click(object sender, EventArgs e)
        {
            _AllowWork.Stop();
        }

        //=//

        private readonly IList<IDisposable> _ToDispose;

        public void Cleanup()
        {
            _ToDispose.ToList().ForEach(o => o.Dispose());
        }
    }
}
