﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using System.Reactive.Concurrency;
using System.Reactive.Linq;

namespace Property.Windows
{
    /// <summary>
    /// Contains words that extend the description 
    /// of a property's reload process.
    /// </summary>
    public static class ReloaderPrototypeExtensionsReactiveWindows
    {
        /// <summary>
        /// <para>... if the call passes through the throttle (when) ...</para>
        /// <para>( ! ) Don't forget to dispose of what you create with this word.</para>
        /// </summary>
        /// <param name="dueTime">Tell me how long I should wait for another call
        /// before letting this one pass.</param>
        /// <param name="testMode">Tell me if I have to ignore this word.</param>
        public static IReloaderPrototype<T> Throttle<T>(this IReloaderPrototype<T> source, TimeSpan dueTime, bool testMode = false)
        {
            return testMode ? source :
                source.Filtered(o => o.Throttle(dueTime)
                .ObserveOn(DispatcherScheduler.Current));
        }
    }
}
