﻿using System;
using System.Reactive.Subjects;

namespace Property.Reloaders
{
    public partial class FilterReloaderDecoratorPrototype<T>
    {
        private class DecoratorInstance : ReloaderDecorator<T>, IReloader
        {
            public DecoratorInstance(IReloaderPrototype<T> compositionBasePrototype,
                Func<T> getValue, Action<T> applyValue, Action<bool> applyStatus,
                Func<IObservable<object>, IObservable<object>> filter)
                : base(compositionBasePrototype, getValue, applyValue, applyStatus)
            {
                if (filter == null) throw new ArgumentNullException("filter");

                _InputRhythm = new Subject<object>();
                _ActionSubscription = filter(_InputRhythm).Subscribe(_ => _CompositionBase.Go());
            }

            //=//

            private readonly ISubject<object> _InputRhythm;
            private readonly IDisposable _ActionSubscription;

            public override void Go()
            {
                _InputRhythm.OnNext(null);
            }

            public override void Dispose()
            {
                _ActionSubscription.Dispose();
                base.Dispose();
            }
        }
    }
}
