﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Property.Reloaders;

namespace Property.Test.Reloaders
{
    [TestClass]
    public class ThrottleReloaderDecoratorPrototypeTest : CallbackReloaderDecoratorPrototypeTestInfrastructure<object>
    {
        [TestMethod]
        public void ThrottleReloaderDecoratorReloadTest()
        {
            var sutDecorator = new ThrottleReloaderDecoratorPrototype<object>(ReloaderPrototypeMock.Object, TimerPrototypeMock.Object)
                .Create(() => null, _ => { }, __ => { });
            sutDecorator.Go();
            VerifyReload(1);
            VerifyTimerStart(1);
            VerifyTimerStop(2);
        }

        [TestMethod]
        public void ThrottleReloaderDecoratorDisposeTest()
        {
            var sutDecorator = new ThrottleReloaderDecoratorPrototype<object>(ReloaderPrototypeMock.Object, TimerPrototypeMock.Object)
                .Create(() => null, _ => { }, __ => { });
            var reference = new WeakReference(sutDecorator, true);

            sutDecorator.Go();
            sutDecorator.Dispose();
            sutDecorator = null;
            GC.Collect();

            Assert.IsNull(reference.Target);
            VerifyDispose();
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void ThrottleReloaderDecoratorNullTimerPrototypeTest()
        {
            new ThrottleReloaderDecoratorPrototype<object>(ReloaderPrototypeMock.Object, null)
                .Create(() => null, _ => { }, __ => { });
        }

        [TestMethod]
        [ExpectedException(typeof(NotSupportedException))]
        public void ThrottleReloaderDecoratorNullTimerTest()
        {
            new ThrottleReloaderDecoratorPrototype<object>(ReloaderPrototypeMock.Object, NullTimerPrototypeMock.Object)
                .Create(() => null, _ => { }, __ => { });
        }
    }
}

