﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Property.Workers;
using System;
using System.ComponentModel;

namespace Property.Test.Workers
{
    [TestClass]
    public class ForegroundWorkerPrototypeTest : WorkerTestInfrastructure<object>
    {
        private readonly object _TestValue = new object();
        protected override object TestValue { get { return base.TestValue; } }

        [TestMethod]
        public void ForegroundWorkerDoWorkTest()
        {
            var sutWorker = new ForegroundWorkerPrototype().Create();

            sutWorker.DoWork += DoWork;
            sutWorker.RunWorkerCompleted += RunWorkerCompleted;

            sutWorker.RunWorkerAsync();

            Assert.IsFalse(sutWorker.IsBusy);
            VerifyCompleted();
        }

        [TestMethod]
        public void ForegroundWorkerDoWorkExceptionTest()
        {
            var sutWorker = new ForegroundWorkerPrototype().Create();

            sutWorker.DoWork += DoWorkException;
            sutWorker.RunWorkerCompleted += RunWorkerCompleted;

            sutWorker.RunWorkerAsync();

            Assert.IsFalse(sutWorker.IsBusy);
            VerifyCompletedException();
        }

        [TestMethod]
        public void ForegroundWorkerDisposeTest()
        {
            var sutWorker = new ForegroundWorkerPrototype().Create();
            var reference = new WeakReference(sutWorker, true);
            sutWorker.DoWork += DoWorkException;
            sutWorker.RunWorkerCompleted += RunWorkerCompleted;
            sutWorker.RunWorkerAsync();

            sutWorker.Dispose();
            sutWorker = null;

            GC.Collect();
            Assert.IsNull(reference.Target);
        }
    }
}
