﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Property.Objects;
using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace Property.Test.Objects
{
    [TestClass]
    public class CallGetPropertyPrototypeTest : DelegatePropertyPrototypeTestInfrastructure<object>
    {
        private readonly object _TestValue = new object();
        protected override object TestValue
        {
            get { return _TestValue; }
        }

        // Get properties with their Value properties bound to UI
        // won't work properly with reloaders that run every time
        // they are called.
        //
        // Consider the following scenario:
        // (1) UI reads Value-> reload Triggered-> new value applied-> 
        // PropertyChanged("Value") raised-> (1) Infinite loop.
        //
        // The following reloader runs on every odd call.
        private bool CanReloadState;
        protected override void DoReloaderBehaviour(System.Func<object> getValue, System.Action<object> applyValue, System.Action<bool> applyStatus)
        {
            if (CanReloadState ^= true) base.DoReloaderBehaviour(getValue, applyValue, applyStatus);
            // Use the following code for one time reloading.
            //if (CanReloadState ? false : CanReloadState ^= true)
            //    base.DoReloaderBehaviour(getValue, applyValue, applyStatus);
        }

        [TestMethod]
        public void CommandGetPropertyReloadTest()
        {
            object getResult = null;

            var sutProperty = new CallGetPropertyPrototype<object>(
                        ReloaderPrototypeMock.Object, GetValueParameterized)
                        .Create(null);

            SetupEvents(sutProperty);

            // Imitate binding
            sutProperty.PropertyChanged += (o, e) =>
            {
                if (e.PropertyName == "Value") getResult = sutProperty.Value;
            };

            Assert.AreEqual(default(object), sutProperty.DebugValue);
            Assert.AreEqual(default(bool), sutProperty.IsLoading);

            sutProperty.Go();

            Assert.AreEqual(TestValue, getResult);
            Assert.AreEqual(TestValue, sutProperty.DebugValue);
            Assert.AreEqual(TestStatus, sutProperty.IsLoading);

            VerifyEvents();
            // 2 times including the call rejected by the reloader
            VerifyReload(2);
        }
    }
}
