﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using System.Reactive.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Property.Reloaders;

namespace Property.Test.Reloaders
{
    [TestClass]
    public class FilterReloaderDecoratorPrototypeTest : ReloaderDecoratorPrototypeTestInfrastructure<object>
    {
        [TestMethod]
        public void FilterReloaderDecoratorReloadTest()
        {
            var sutDecorator = new FilterReloaderDecoratorPrototype<object>(ReloaderPrototypeMock.Object, o => o.Take(1))
                .Create(() => null, _ => { }, __ => { });
            sutDecorator.Go();
            VerifyReload(1);
            sutDecorator.Go();
            VerifyReload(1);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void FilterReloaderDecoratorNullFilterTest()
        {
            new FilterReloaderDecoratorPrototype<object>(ReloaderPrototypeMock.Object, null)
                .Create(() => null, _ => { }, __ => { });
        }

        [TestMethod]
        public void FilterReloaderDecoratorDisposeTest()
        {
            var sutDecorator = new FilterReloaderDecoratorPrototype<object>(ReloaderPrototypeMock.Object, o => o.Take(1))
                .Create(() => null, _ => { }, __ => { });
            var reference = new WeakReference(sutDecorator, true);
            sutDecorator.Go();
            sutDecorator.Go();
            sutDecorator.Dispose();
            sutDecorator = null;
            GC.Collect();
            Assert.IsNull(reference.Target);
        }
    }
}
