﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Moq;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Property.Test
{
    public abstract class SequenceConveyorTestInfrastructure<T>
    {

        private IEnumerable<string> ResultMethodCallStrings = Enumerable.Empty<string>();

        protected virtual IEnumerable<string> TestMethodCallStrings
        { get { return new[] { "GetSequence" }; } }

        protected virtual T TestValue { get { return default(T); } }

        protected virtual IObservable<T> GetSequence(Func<T> source)
        {
            ResultMethodCallStrings = ResultMethodCallStrings.Concat(new[] { "GetSequence" });
            return SequenceMock.Object;
        }

        protected virtual IObservable<T> ModifySequence(IObservable<T> source)
        {
            ResultMethodCallStrings = ResultMethodCallStrings.Concat(new[] { "ModifySequence" });
            return source;
        }

        protected virtual T GetValue()
        {
            ResultMethodCallStrings = ResultMethodCallStrings.Concat(new[] { "GetValue" });
            return default(T);
        }

        private Mock<IObservable<T>> SequenceMockState;
        protected Mock<IObservable<T>> SequenceMock
        {
            get
            {
                return SequenceMockState ??
                    (SequenceMockState = new Mock<IObservable<T>>(MockBehavior.Strict));
            }
        }

        protected virtual void VerifyCreate()
        {
            new[] { ResultMethodCallStrings.SequenceEqual(TestMethodCallStrings) }
                .Where(o => !o)
                .ToList()
                .ForEach(o => { throw new Exception(); });
        }
    }
}
