﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace Showcase.LoadByWire
{
    // You'll need something like this for validation.
    // The question is where you want to place it... Oh, and how you'll implement it...
    public static class ValidationService
    {
        public static List<string> GetPassportNrErrors(string passportNr)
        {
            if (string.IsNullOrWhiteSpace(passportNr)) return new List<string>() { "Empty" };

            var res = new List<string>(2);
            if (passportNr.Length > 30) res.Add("TooLong");

            return res;
        }

        public static List<string> GetFirstNameErrors(string firstName)
        {
            if (string.IsNullOrWhiteSpace(firstName)) return new List<string>() { "Empty" };

            var res = new List<string>(2);
            if (firstName.Length > 30) res.Add("TooLong");
            if (!Regex.IsMatch(firstName, @"^([ \u00c0-\u01ffa-zA-Z'\-])+$")) res.Add("IllegalCharacters");

            return res;
        }

        public static List<string> GetLastNameErrors(string lastName)
        {
            if (string.IsNullOrWhiteSpace(lastName)) return new List<string>() { "Empty" };

            var res = new List<string>(2);
            if (lastName.Length > 30) res.Add("TooLong");
            if (!Regex.IsMatch(lastName, @"^([ \u00c0-\u01ffa-zA-Z'\-])+$")) res.Add("IllegalCharacters");

            return res;
        }

        public static List<string> GetCompanyErrors(string company)
        {
            if (string.IsNullOrWhiteSpace(company)) return new List<string>() { "Empty" };

            var res = new List<string>(2);
            if (company.Length > 30) res.Add("TooLong");

            return res;
        }

        public static List<string> GetEmailErrors(string email)
        {
            if (string.IsNullOrWhiteSpace(email)) return new List<string>() { "Empty" };

            // Copied from Phil Haack's site
            // http://haacked.com/archive/2007/08/21/i-knew-how-to-validate-an-email-address-until-i.aspx
            // Is used here for demonstration purposes only

            string pattern = @"^(?!\.)(""([^""\r\\]|\\[""\r\\])*""|"
                + @"([-a-z0-9!#$%&'*+/=?^_`{|}~]|(?<!\.)\.)*)(?<!\.)"
                + @"@[a-z0-9][\w\.-]*[a-z0-9]\.[a-z][a-z\.]*[a-z]$";

            var res = new List<string>(2);
            if (!Regex.IsMatch(email, pattern)) res.Add("IllegalFormat");

            return res;
        }
    }
}
