﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using System.ComponentModel;

namespace Property.Reloaders
{
    public class CallbackReloader<T> : OutReloader<T>
    {
        public CallbackReloader(IFunctionWorkerPrototype<T> workerPrototype, Func<T> getValue, Action<T> applyValue, Action<bool> applyStatus)
            : base(applyValue, applyStatus)
        {
            if (workerPrototype == null) throw new ArgumentNullException("workerPrototype");

            _Worker = workerPrototype.Create(getValue);

            if (_Worker == null)
                throw new NotSupportedException(StringsCallbacks.CANT_OBTAIN_WORKER);

            _Worker.RunWorkerCompleted += ResultReceived;
        }

        //=//

        private readonly IWorker _Worker;

        public override void Go()
        {
            _ApplyStatus(true);
            _Worker.RunWorkerAsync();
        }

        private void ResultReceived(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null) throw e.Error;

            _ApplyStatus(false);
            _ApplyValue((T)e.Result);
        }

        public override void Dispose()
        {
            _Worker.RunWorkerCompleted -= ResultReceived;
            _Worker.Dispose();
            base.Dispose();
        }
    }
}
