﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Property.Reloaders;

namespace Property.Test.Reloaders
{
    [TestClass]
    public class CallbackReloaderPrototypeTest : CallbackReloaderPrototypeTestInfrastructure<object>
    {
        private readonly object _TestValue = new object();
        protected override object TestValue
        {
            get { return _TestValue; }
        }

        private bool TestIsBusyState = false;
        protected override bool TestIsBusy { get { return TestIsBusyState; } }

        [TestMethod]
        public void CallbackReloaderReloadTest()
        {
            var sutReloader = new CallbackReloaderPrototype<object>(FunctionWorkerPrototypeMock.Object).Create(GetValue, ApplyValue, ApplyStatus);

            SetupReload();
            sutReloader.Go();

            VerifyReload();
        }

        [TestMethod]
        public void CallbackReloaderReloadBusyTest()
        {
            var sutReloader = new CallbackReloaderPrototype<object>(FunctionWorkerPrototypeMock.Object).Create(GetValue, ApplyValue, ApplyStatus);

            SetupReload();
            TestIsBusyState = true;
            sutReloader.Go();

            VerifyReload();
        }

        [TestMethod]
        public void CallbackReloaderReloadExceptionTest()
        {
            try
            {
                var sutReloader = new CallbackReloaderPrototype<object>(FunctionWorkerPrototypeMock.Object).Create(GetValue, ApplyValue, ApplyStatus);

                SetupReloadException();
                sutReloader.Go();
            }
            catch (ApplicationException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void CallbackReloaderDisposeTest()
        {
            var sutReloader = new CallbackReloaderPrototype<object>(FunctionWorkerPrototypeMock.Object).Create(GetValue, ApplyValue, ApplyStatus);
            var reference = new WeakReference(sutReloader, true);

            SetupReload();

            sutReloader.Go();
            TestIsBusyState = true;
            sutReloader.Go();
            sutReloader.Dispose();
            sutReloader = null;
            GC.Collect();

            Assert.IsNull(reference.Target);
            VerifyDispose();
        }

        [TestMethod]
        public void CallbackReloaderNullWorkerPrototypeTest()
        {
            try
            {
                new CallbackReloaderPrototype<object>(null).Create(GetValue, ApplyValue, ApplyStatus);
            }
            catch (ArgumentNullException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void CallbackReloaderNullWorkerTest()
        {
            try
            {
                new CallbackReloaderPrototype<object>(NullFunctionWorkerPrototypeMock.Object).Create(GetValue, ApplyValue, ApplyStatus);
            }
            catch (NotSupportedException) { return; }
            Assert.Fail();
        }
    }
}
