﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Property.Workers;
using System.Collections.Generic;
using System.ComponentModel;

namespace Property.Test.Workers
{
    [TestClass]
    public class CatchFunctionWorkerDecoratorPrototypeTest : FunctionWorkerDecoratorPrototypeTestInfrastructure<object>
    {
        private IEnumerable<string> TestEventHandlerCallStringsState;
        protected override IEnumerable<string> TestEventHandlerCallStrings
        { get { return TestEventHandlerCallStringsState; } }

        private bool TestIsBusyState = false;
        protected override bool TestIsBusy { get { return TestIsBusyState; } }

        [TestMethod]
        public void CatchFunctionWorkerDecoratorNoExceptionTest()
        {
            var sutDecorator = new CatchFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, TestHandler).Create(() => null);

            TestEventHandlerCallStringsState = new[] { "RunWorkerCompleted" };

            sutDecorator.RunWorkerCompleted += RunWorkerCompleted;
            sutDecorator.RunWorkerAsync();
            TestIsBusyState = true;
            sutDecorator.RunWorkerAsync();

            WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(TestValue, null, false));
            
            VerifyCompleted();
        }

        [TestMethod]
        public void CatchFunctionWorkerDecoratorExceptionTest()
        {
            var sutDecorator = new CatchFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, TestHandler).Create(() => null);

            TestEventHandlerCallStringsState = new[] { "TestHandler" };

            sutDecorator.RunWorkerCompleted += RunWorkerCompleted;
            sutDecorator.RunWorkerAsync();
            WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(null, new ApplicationException(), false));
            VerifyCompletedException();
        }

        [TestMethod]
        public void CatchFunctionWorkerDecoratorNullHandlerTest()
        {
            try
            {
                new CatchFunctionWorkerDecoratorPrototype<object>(
                    FunctionWorkerPrototypeMock.Object, null).Create(() => null);
            }
            catch (ArgumentNullException) { }
        }
    }
}
