﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Property.Workers;

namespace Property.Test.Workers
{
    [TestClass]
    public class FunctionWorkerDecoratorPrototypeTest : FunctionWorkerDecoratorPrototypeTestInfrastructure<object>
    {
        [TestMethod]
        public void FunctionWorkerDecoratorDisposeTest()
        {
            var sutDecorator = new CatchFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, e => { }).Create(() => null);
            var reference = new WeakReference(sutDecorator, true);

            sutDecorator.RunWorkerCompleted += RunWorkerCompleted;

            sutDecorator.RunWorkerAsync();
            sutDecorator.Dispose();
            sutDecorator = null;
            GC.Collect();

            Assert.IsNull(reference.Target);
            VerifyDispose();
        }

        [TestMethod]
        public void FunctionWorkerDecoratorIsBusyTest()
        {
            var sutDecorator = new CatchFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, e => { }).Create(() => null);

            var testIsBusy = sutDecorator.IsBusy;

            VerifyIsBusy();
        }

        [TestMethod]
        public void FunctionWorkerDecoratorNullWorkerPrototypeTest()
        {
            try
            {
                new CatchFunctionWorkerDecoratorPrototype<object>(
                    null, e => { }).Create(() => null);
            }
            catch (ArgumentNullException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void FunctionWorkerDecoratorNullWorkerTest()
        {
            try
            {
                new CatchFunctionWorkerDecoratorPrototype<object>(
                    NullFunctionWorkerPrototypeMock.Object, e => { }).Create(() => null);
            }
            catch (NotSupportedException) { return; }
            Assert.Fail();
        }
    }
}
