﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Property.Workers;
using System.ComponentModel;
using System.Collections.Generic;

namespace Property.Test.Workers
{
    [TestClass]
    public class FunctionWorkerPrototypeTest : FunctionWorkerPrototypeTestInfrastructure<object>
    {
        protected override IEnumerable<string> TestEventHandlerCallStrings
        { get { return new[] { "RunWorkerCompleted" }; } }

        private readonly object _TestValue = new object();
        protected override object TestValue { get { return _TestValue; } }

        private bool TestIsBusyState = false;
        protected override bool TestIsBusy { get { return TestIsBusyState; } }

        [TestMethod]
        public void FunctionWorkerIsBusyTest()
        {
            var sutWorker = new FunctionWorkerPrototype<object>(WorkerPrototypeMock.Object).Create(() => null);

            Assert.AreEqual(TestIsBusy, sutWorker.IsBusy);
            VerifyIsBusy();
        }

        [TestMethod]
        public void FunctionWorkerDoWorkTest()
        {
            var sutWorker = new FunctionWorkerPrototype<object>(WorkerPrototypeMock.Object).Create(() => null);

            sutWorker.RunWorkerCompleted += RunWorkerCompleted;

            sutWorker.RunWorkerAsync();

            WorkerMock.Raise(o => o.DoWork += null, new DoWorkEventArgs(null));
            WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(TestValue, null, false));

            VerifyCompleted();
            VerifyRunWorkerAsync();
        }

        [TestMethod]
        public void FunctionWorkerDoWorkExceptionTest()
        {
            var sutWorker = new FunctionWorkerPrototype<object>(WorkerPrototypeMock.Object).Create(() => null);

            sutWorker.RunWorkerCompleted += RunWorkerCompleted;

            sutWorker.RunWorkerAsync();

            WorkerMock.Raise(o => o.DoWork += null, new DoWorkEventArgs(null));
            WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(null, new ApplicationException(), false));

            VerifyCompletedException();
            VerifyRunWorkerAsync();
        }

        [TestMethod]
        public void FunctionWorkerDisposeTest()
        {
            var sutWorker = new FunctionWorkerPrototype<object>(WorkerPrototypeMock.Object).Create(() => null);
            var reference = new WeakReference(sutWorker, true);

            sutWorker.RunWorkerCompleted += RunWorkerCompleted;
            sutWorker.RunWorkerAsync();
            TestIsBusyState = true;
            sutWorker.RunWorkerAsync();

            WorkerMock.Raise(o => o.DoWork += null, new DoWorkEventArgs(null));
            WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(null, null, false));

            sutWorker.Dispose();
            sutWorker = null;

            GC.Collect();
            Assert.IsNull(reference.Target);
            VerifyDisposeCreate(2);
        }

        [TestMethod]
        public void FunctionWorkerNullWorkerPrototypeTest()
        {
            try
            {
                new FunctionWorkerPrototype<object>(null).Create(() => null);
            }
            catch (ArgumentNullException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void FunctionWorkerNullWorkerTest()
        {
            try
            {
                new FunctionWorkerPrototype<object>(NullWorkerPrototypeMock.Object).Create(() => null);
            }
            catch (NotSupportedException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void FunctionWorkerNullGetValueTest()
        {
            try
            {
                new FunctionWorkerPrototype<object>(WorkerPrototypeMock.Object).Create(null);
            }
            catch (ArgumentNullException) { return; }
            Assert.Fail();
        }
    }
}
