﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Property.Workers;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

namespace Property.Test.Workers
{
    [TestClass]
    public class RetryFunctionWorkerDecoratorPrototypeTest : FunctionWorkerDecoratorPrototypeTestInfrastructure<object>
    {
        private readonly object _TestValue = new object();
        protected override object TestValue{ get { return _TestValue; } }

        private IEnumerable<string> TestEventHandlerCallStringsState;
        protected override IEnumerable<string> TestEventHandlerCallStrings
        { get { return TestEventHandlerCallStringsState; } }

        private IEnumerable<object> TestValuesState;
        protected override IEnumerable<object> TestValues
        { get { return TestValuesState; } }

        private bool TestIsBusyState = false;
        protected override bool TestIsBusy { get { return TestIsBusyState; } }

        private readonly int _TestRetryCount = 4;
        protected int TestRetryCount { get { return _TestRetryCount; } }

        private readonly int _TestRepeat = 3;
        protected int TestRepeat { get { return _TestRepeat; } }

        [TestMethod]
        public void RetryFunctionWorkerDecoratorNoExceptionTest()
        {
            var sutDecorator = new RetryFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, TestRetryCount).Create(() => null);

            TestEventHandlerCallStringsState = Enumerable.Repeat("RunWorkerCompleted", TestRepeat);
            TestValuesState = Enumerable.Repeat(TestValue, TestRepeat);

            sutDecorator.RunWorkerCompleted += RunWorkerCompleted;

            Enumerable.Repeat(TestValue, TestRepeat).ToList().ForEach(v =>
                {
                    sutDecorator.RunWorkerAsync();
                    Enumerable.Repeat(new ApplicationException(), TestRetryCount - 1).ToList().ForEach(e =>
                        WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(null, e, false)));
                    WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(v, null, false));
                });
            
            VerifyCompleted();
            VerifyWorkerCreate();
            VerifyWorkerCall(TestRetryCount * TestRepeat);
        }

        [TestMethod]
        public void RetryFunctionWorkerDecoratorExceptionTest()
        {
            var sutDecorator = new RetryFunctionWorkerDecoratorPrototype<object>(
                FunctionWorkerPrototypeMock.Object, TestRetryCount).Create(() => null);

            TestEventHandlerCallStringsState = new[] { "RunWorkerCompleted" };

            sutDecorator.RunWorkerCompleted += RunWorkerCompleted;

            sutDecorator.RunWorkerAsync();
            Enumerable.Repeat(new ApplicationException(), TestRetryCount).ToList().ForEach(e =>
                WorkerMock.Raise(o => o.RunWorkerCompleted += null, new RunWorkerCompletedEventArgs(null, e, false)));

            VerifyCompletedException();
            VerifyWorkerCreate();
            VerifyWorkerCall(TestRetryCount);
        }

        [TestMethod]
        public void RetryFunctionWorkerDecoratorZeroRetryCountTest()
        {
            try
            {
                new RetryFunctionWorkerDecoratorPrototype<object>(
                    FunctionWorkerPrototypeMock.Object, 0).Create(() => null);
            }
            catch (ArgumentOutOfRangeException) { return; }
            Assert.Fail();
        }
    }
}
