﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Moq;
using Property.Reloaders;
using System;

namespace Property.Test
{
    public abstract class ReloaderDecoratorPrototypeTestInfrastructure<T>
    {
        private Mock<IReloader> ReloaderMockState;
        protected Mock<IReloader> ReloaderMock
        {
            get
            {
                if (ReloaderMockState == null)
                {
                    ReloaderMockState = new Mock<IReloader>(MockBehavior.Strict);
                    ReloaderMockState.Setup(o => o.Go());
                    ReloaderMockState.Setup(o => o.Dispose());
                }
                return ReloaderMockState;
            }
        }

        private Mock<IReloaderPrototype<T>> ReloaderPrototypeMockState;
        protected Mock<IReloaderPrototype<T>> ReloaderPrototypeMock
        {
            get
            {
                if (ReloaderPrototypeMockState == null)
                {
                    ReloaderPrototypeMockState = new Mock<IReloaderPrototype<T>>(MockBehavior.Strict);
                    ReloaderPrototypeMockState.Setup(o => o.Create(
                        It.IsAny<Func<T>>(),
                        It.IsAny<Action<T>>(),
                        It.IsAny<Action<bool>>()))
                        .Returns(ReloaderMock.Object);
                }
                return ReloaderPrototypeMockState;
            }
        }

        private Mock<IReloaderPrototype<T>> NullReloaderPrototypeMockState;
        protected Mock<IReloaderPrototype<T>> NullReloaderPrototypeMock
        {
            get
            {
                if (NullReloaderPrototypeMockState == null)
                {
                    NullReloaderPrototypeMockState = new Mock<IReloaderPrototype<T>>(MockBehavior.Strict);
                    NullReloaderPrototypeMockState.Setup(o => o.Create(
                        It.IsAny<Func<T>>(),
                        It.IsAny<Action<T>>(),
                        It.IsAny<Action<bool>>()))
                        .Returns<IReloader>(null);
                }
                return NullReloaderPrototypeMockState;
            }
        }

        //=//

        protected virtual void VerifyCreate()
        {
            ReloaderPrototypeMock.Verify(o => o.Create(
                It.IsAny<Func<T>>(),
                It.IsAny<Action<T>>(),
                It.IsAny<Action<bool>>()),
                Times.Once());
        }

        protected virtual void VerifyReload(int times)
        {
            VerifyCreate();
            ReloaderMock.Verify(o => o.Go(), Times.Exactly(times));
        }

        protected virtual void VerifyDispose()
        {
            VerifyCreate();
            ReloaderMock.Verify(o => o.Dispose(), Times.Once());
        }
    }
}
