﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Property.Reloaders;

namespace Property.Test.Reloaders
{
    [TestClass]
    public class ReactiveReloaderPrototypeTest : ReactiveReloaderPrototypeTestInfrastructure<object>
    {
        private readonly object _TestValue = new object();
        protected override object TestValue
        {
            get { return _TestValue; }
        }

        [TestMethod]
        public void ReactiveReloaderReloadTest()
        {
            var sutReloader = new ReactiveReloaderPrototype<object>(SequenceConveyorMock.Object).Create(GetValue, ApplyValue, ApplyStatus);

            sutReloader.Go();

            VerifyReload();
        }

        [TestMethod]
        public void ReactiveReloaderNullSequenceConveyorTest()
        {
            try
            {
                new ReactiveReloaderPrototype<object>(null).Create(GetValue, ApplyValue, ApplyStatus);
            }
            catch (ArgumentNullException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void ReactiveReloaderNullSequenceTest()
        {
            try
            {
                new ReactiveReloaderPrototype<object>(NullSequenceConveyorMock.Object).Create(GetValue, ApplyValue, ApplyStatus);
            }
            catch (NotSupportedException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void ReactiveReloaderNullGetValueTest()
        {
            new ReactiveReloaderPrototype<object>(SequenceConveyorMock.Object).Create(null, ApplyValue, ApplyStatus);
        }

        [TestMethod]
        public void ReactiveReloaderNullApplyValueTest()
        {
            try
            {
                new ReactiveReloaderPrototype<object>(SequenceConveyorMock.Object).Create(GetValue, null, ApplyStatus);
            }
            catch (ArgumentNullException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void ReactiveReloaderNullApplyStatusTest()
        {
            try
            {
                new ReactiveReloaderPrototype<object>(SequenceConveyorMock.Object).Create(GetValue, ApplyValue, null);
            }
            catch (ArgumentNullException) { return; }
            Assert.Fail();
        }

        [TestMethod]
        public void ReactiveReloaderDisposeTest()
        {
            var sutReloader = new ReactiveReloaderPrototype<object>(OpenSequenceConveyorMock.Object).Create(GetValue, ApplyValue, ApplyStatus);
            var reference = new WeakReference(sutReloader, true);
            sutReloader.Go();
            sutReloader.Go();
            sutReloader.Dispose();
            sutReloader = null;
            GC.Collect();
            Assert.IsNull(reference.Target);
        }
    }
}

