﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Moq;
using System;
using System.Reactive;
using System.Reactive.Disposables;
using System.Reactive.Linq;

namespace Property.Test
{
    public abstract class ReactiveReloaderPrototypeTestInfrastructure<T> 
        : ReloaderPrototypeTestInfrastructure<T>
    {
        private Mock<ISequenceConveyor<T>> SequenceConveyorMockState;
        protected Mock<ISequenceConveyor<T>> SequenceConveyorMock
        {
            get
            {
                if (SequenceConveyorMockState == null)
                {
                    SequenceConveyorMockState = new Mock<ISequenceConveyor<T>>(MockBehavior.Strict);
                    SequenceConveyorMockState.Setup(o => o.Create(It.IsAny<Func<T>>()))
                        .Returns<Func<T>>(
                        o => Observable.Defer<T>(() => Observable.Return<T>(o()))
                        );
                }
                return SequenceConveyorMockState;
            }
        }

        private Mock<ISequenceConveyor<T>> NullSequenceConveyorMockState;
        protected Mock<ISequenceConveyor<T>> NullSequenceConveyorMock
        {
            get
            {
                if (NullSequenceConveyorMockState == null)
                {
                    NullSequenceConveyorMockState = new Mock<ISequenceConveyor<T>>(MockBehavior.Strict);
                    NullSequenceConveyorMockState.Setup(o => o.Create(It.IsAny<Func<T>>()))
                        .Returns<IObservable<T>>(null);
                }
                return NullSequenceConveyorMockState;
            }
        }

        private Mock<ISequenceConveyor<T>> OpenSequenceConveyorMockState;
        protected Mock<ISequenceConveyor<T>> OpenSequenceConveyorMock
        {
            get
            {
                if (OpenSequenceConveyorMockState == null)
                {
                    OpenSequenceConveyorMockState = new Mock<ISequenceConveyor<T>>(MockBehavior.Strict);
                    OpenSequenceConveyorMockState.Setup(o => o.Create(It.IsAny<Func<T>>()))
                        .Returns<Func<T>>(_ => Observable.Never<T>());
                }
                return OpenSequenceConveyorMockState;
            }
        }
    }
}
