﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Microsoft.VisualStudio.TestPlatform.UnitTestFramework;
using Property.Objects;
using System;
using System.Collections.Generic;

namespace Property.Test.Objects
{
    [TestClass]
    public class ValidationInputPrototypePropertyTest : ValidationPropertyPrototypeTestInfrastructure<object>
    {
        protected override IEnumerable<string> TestEventStrings
        { get { return new[] { 
            "Changing'Input'", 
            "Changed'Input'", 
            "Changing'IsLoading'", 
            "Changed'IsLoading'", 
            "Changing'Value'", 
            "Changed'Value'" }; } }

        [TestMethod]
        public void ValidationInputPropertyReloadTest()
        {
            var testValue = new object();

            var sutProperty = new ValidationInputPropertyPrototype<object>(
                        ReloaderPrototypeMock.Object, o => TestErrorStrings).Create(null);

            SetupValidation(sutProperty, "Input");
            SetupEvents(sutProperty);

            Assert.AreEqual(default(object), sutProperty.Input);
            Assert.AreEqual(default(object), sutProperty.Value);
            Assert.AreEqual(default(object), sutProperty.DebugValue);
            Assert.AreEqual(default(bool), sutProperty.IsLoading);

            sutProperty.Input = testValue;

            Assert.AreEqual(testValue, sutProperty.Value);
            Assert.AreEqual(testValue, sutProperty.DebugValue);
            Assert.AreEqual(TestStatus, sutProperty.IsLoading);

            VerifyValidation();
            VerifyEvents();
            VerifyReload();
        }

        [TestMethod]
        public void ValidationInputPropertyNullValidationTest()
        {
            try
            {
                new ValidationInputPropertyPrototype<object>(
                            ReloaderPrototypeMock.Object, null).Create(null);
            }
            catch (ArgumentNullException) { return; }
            Assert.Fail();
        }
    }
}
