﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Microsoft.Phone.Controls;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;

namespace Showcase.LoadAsYouScroll
{
    public class FireWhenScrolledBehaviour : Behavior<ScrollViewer>
    {
        #region Command

        public static readonly DependencyProperty CommandProperty =
             DependencyProperty.Register("Command", typeof(ICommand),
             typeof(FireWhenScrolledBehaviour), null);

        public ICommand Command
        {
            get { return (ICommand)GetValue(CommandProperty); }
            set { SetValue(CommandProperty, value); }
        }

        #endregion Command

        #region Private Dependency Properties

        private static readonly DependencyProperty VerticalOffsetProperty =
            DependencyProperty.Register("VerticalOffset", typeof(double),
            typeof(FireWhenScrolledBehaviour), new PropertyMetadata(new PropertyChangedCallback((o, e) =>
            {
                DoSignal(o as FireWhenScrolledBehaviour);
            })));

        private static readonly DependencyProperty ExtentHeightProperty =
            DependencyProperty.Register("ExtentHeight", typeof(double),
            typeof(FireWhenScrolledBehaviour), new PropertyMetadata(new PropertyChangedCallback((o, e) =>
            {
                DoSignal(o as FireWhenScrolledBehaviour);
            })));

        private static readonly DependencyProperty ViewportHeightProperty =
            DependencyProperty.Register("ViewportHeight", typeof(double),
            typeof(FireWhenScrolledBehaviour), new PropertyMetadata(new PropertyChangedCallback((o, e) =>
            {
                DoSignal(o as FireWhenScrolledBehaviour);
            })));

        #endregion Private Dependency Properties

        private static void DoSignal(FireWhenScrolledBehaviour o)
        {
            DoSignal((double)o.GetValue(VerticalOffsetProperty), (double)o.GetValue(ExtentHeightProperty), 
                (double)o.GetValue(ViewportHeightProperty), (ICommand)o.GetValue(CommandProperty));
        }

        private static void DoSignal(double verticalOffset, double extentHeight, double viewportHeight, ICommand command)
        {
            if (verticalOffset <= extentHeight - viewportHeight - 1) return;
            if (command == null || !command.CanExecute(null)) return;

            command.Execute(null);
        }

        protected override void OnAttached()
        {
            AssociatedObject.Loaded += OnLoaded;
        }

        protected override void OnDetaching()
        {
            AssociatedObject.Loaded -= OnLoaded;
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Binding verticalOffsetBinding = new Binding();
            verticalOffsetBinding.Source = AssociatedObject;
            verticalOffsetBinding.Path = new PropertyPath("VerticalOffset");
            verticalOffsetBinding.Mode = BindingMode.OneWay;
            BindingOperations.SetBinding(this, VerticalOffsetProperty, verticalOffsetBinding);

            Binding extentHeightBinding = new Binding();
            extentHeightBinding.Source = AssociatedObject;
            extentHeightBinding.Path = new PropertyPath("ExtentHeight");
            extentHeightBinding.Mode = BindingMode.OneWay;
            BindingOperations.SetBinding(this, ExtentHeightProperty, extentHeightBinding);

            Binding viewportHeightBinding = new Binding();
            viewportHeightBinding.Source = AssociatedObject;
            viewportHeightBinding.Path = new PropertyPath("ViewportHeight");
            viewportHeightBinding.Mode = BindingMode.OneWay;
            BindingOperations.SetBinding(this, ViewportHeightProperty, viewportHeightBinding);
        }
    }
}
