﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Property;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Windows.Input;

namespace Showcase.LoadAsYouScroll
{
    public class MainViewModel : IDisposable
    {
        public MainViewModel()
        {
            var test = Reloadable<string>.On().Worker().Each().Call(_ =>
                {
                    var loadTestResult = ScrollListCheck
                        .CreateRunDisposeCheck<string>(1000);

                    var operationTestResult = ScrollListCheck
                        .OperationCheck<string>();

                    return String.Format("ScrollList tests {0}! It took {1} ms to create, run, and dispose of 1000 ScrollList instances.",
                        (loadTestResult.Item2 && operationTestResult) ? "PASSED" : "FAILED", loadTestResult.Item1.Milliseconds);
                }).Create("Press 'Run Tests' to start ScrollList test series.");

            _RunTestsCommand = new DelegateCommand(_ => test.Go());
            _TestResults = test;

            _ListInAction = new ScrollList<string>(GetIt);

            // The disposal we keep in mind.
            _ToDispose = new[] { (IDisposable)test, _ListInAction };
        }

        //=//

        private readonly ScrollList<string> _ListInAction;
        public ScrollList<string> ListInAction { get { return _ListInAction; } }

        private readonly IProperty<string> _TestResults;
        public IProperty<string> TestResults { get { return _TestResults; } }

        private readonly ICommand _RunTestsCommand;
        public ICommand RunTestsCommand { get { return _RunTestsCommand; } }

        private IEnumerable<string> GetIt(int page)
        {
            Thread.Sleep(1000);
            if (page < 10)
                return Enumerable.Range(page * 10 + 1, 2).Select(o => o.ToString());
            else
                return Enumerable.Empty<string>();
        }

        //=//

        private readonly IList<IDisposable> _ToDispose;

        public void Dispose()
        {
            _ToDispose.ToList().ForEach(o => o.Dispose()); ;
        }
    }
}
