﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Property;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;

namespace Showcase.LoadAsYouScroll
{
    // Implements Load As You Scroll pattern
    public class ScrollList<T> : IDisposable
    {
        /// <param name="getPage">Function that accepts the page number
        /// and returns a page of data.</param>
        public ScrollList(Func<int, IEnumerable<T>> getPage, bool testMode = false)
        {
            // Describe how each property behaves
            var pageNumber = Reloadable<int>.On().Each()
                .Call(o => o + 1).Create();

            var allPagesLoaded = Reloadable<bool>.On().First()
                .Call(_ => true).Create();

            var newPage = Reloadable<IEnumerable<T>>.On().Each()
                .Get(_ => getPage(pageNumber.Value))
                .Create(Enumerable.Empty<T>());

            var list = Reloadable<IEnumerable<T>>.On().Worker(testMode).Odd()
                .CallGet(o => o.Concat(newPage.Value))
                .Create(Enumerable.Empty<T>());

            // Describe how properties call each other
            var newPageRead = newPage.OnChanged(() => newPage.DebugValue.Any() ? (ICallDriven)pageNumber : allPagesLoaded);

            // Describe what should be visible
            _Command = new DelegateCommand(_ =>  list.Go(),__=> !list.IsLoading && !allPagesLoaded.Value);
            _List = list;

            // Describe what should be disposed of
            _ToDispose = new[] { list, newPageRead };
        }

        //=//

        private readonly ICommand _Command;
        public ICommand Command { get { return _Command; } }

        private readonly IProperty<IEnumerable<T>> _List;
        public IProperty<IEnumerable<T>> List { get { return _List; } }

        //=//

        private readonly IList<IDisposable> _ToDispose;

        public void Dispose()
        {
            _ToDispose.ToList().ForEach(o => o.Dispose()); ;
        }
    }
}