﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using System;
using System.Windows.Input;

namespace Showcase.LoadByWire
{
    public class DelegateCommand : ICommand
    {
        public DelegateCommand(Action<object> execute,
                       Func<object, bool> canExecute = null)
        {
            if (execute == null) throw new ArgumentNullException("execute");

            _Execute = execute;
            _CanExecute = canExecute ?? (_ => true);
        }

        //=//

        private readonly Func<object, bool> _CanExecute;
        private readonly Action<object> _Execute;

        public event EventHandler CanExecuteChanged;

        public bool CanExecute(object parameter)
        {
            return _CanExecute(parameter);
        }

        public void Execute(object parameter)
        {
            _Execute(parameter);
        }

        public void OnCanExecuteChanged()
        {
            if (CanExecuteChanged != null)
                CanExecuteChanged(this, EventArgs.Empty);
        }
    }
}
