﻿// Copyright (c) Vitaly Kamiansky. All rights reserved. 
// See License.txt in the project root.

using Property;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Windows.Input;

namespace Showcase.ProgressCounter
{
    public class MainViewModel : IDisposable
    {
        public MainViewModel()
        {
            Func<int> getMinimum = () => 0;
            Func<int> getMaximum = () => 100;
            Func<int> getStep = () => 10;

            var minimum = Reloadable<int>.On().Odd().CallGet(_ => getMinimum()).Create();
            var maximum = Reloadable<int>.On().Odd().CallGet(_ => getMaximum()).Create();

            var counter = new Counter(getMinimum, getMaximum, getStep);
            var allowWork = new Actuator();

            // Declare shortcuts.
            Func<bool> workAllowed = () => allowWork.Power.Value;
            Func<bool> workNotDone = () => counter.Output.Value < getMaximum();
            Func<bool> needReset = () => counter.IsReset.Value;
            Func<bool> needWork = () => needReset() || (workNotDone() && workAllowed());

            Func<int, int> work = o =>
            {
                if (needReset()) return 0;
                Thread.Sleep(1000);
                return o + 1;
            };

            var doWork = Reloadable<int>.On().Worker().Conditional(needWork).Call(work).Create();
            var onDoWork = doWork.OnChanged(() => counter, () => doWork);

            // Assign fields.
            _Start = new DelegateCommand(_ =>
            {
                allowWork.Start();
                counter.Reset();
                doWork.Go();
            });

            _Abort = new DelegateCommand(_ =>
            {
                allowWork.Stop();
                counter.Reset();
                doWork.Go();
            });

            _Resume = new DelegateCommand(_ =>
            {
                allowWork.Start();
                doWork.Go();
            });

            _Pause = new DelegateCommand(_ =>
            {
                allowWork.Stop();
            });

            _Minimum = minimum;
            _Maximum = maximum;
            _Counter = counter;
            _DoWork = doWork;

            // Add disposables to the cleanup list.
            _ToDispose = new[] { doWork, onDoWork };
        }

        private readonly ICommand _Start;
        public ICommand Start { get { return _Start; } }

        private readonly ICommand _Abort;
        public ICommand Abort { get { return _Abort; } }

        private readonly ICommand _Resume;
        public ICommand Resume { get { return _Resume; } }

        private readonly ICommand _Pause;
        public ICommand Pause { get { return _Pause; } }

        private readonly IProperty<int> _Minimum;
        public IProperty<int> Minimum { get { return _Minimum; } }

        private readonly IProperty<int> _Maximum;
        public IProperty<int> Maximum { get { return _Maximum; } }

        private readonly Counter _Counter;
        public Counter Counter { get { return _Counter; } }

        private readonly IProperty<int> _DoWork;
        public IProperty<int> DoWork { get { return _DoWork; } }

        //=//

        private readonly IList<IDisposable> _ToDispose;

        public void Dispose()
        {
            _ToDispose.ToList().ForEach(o => o.Dispose()); ;
        }
    }
}
